#include <iostream>
#include <cstdint>
#include <cmath>
#include <vector>
#include <utility>
#include <iomanip>
#include <algorithm>

struct Vector {
	double x, y;
	Vector(double pX = 0, double pY = 0): x(pX), y(pY) {}
	friend Vector operator+ (const Vector &v1, const Vector &v2) {
		return Vector (v1.x + v2.x, v1.y + v2.y);
	}
	friend Vector operator- (const Vector &v1, const Vector &v2) {
		return Vector (v1.x - v2.x, v1.y - v2.y);
	}
	static double dist(const Vector &v1, const Vector &v2) {
		double dx = v1.x - v2.x;
		double dy = v1.y - v2.y;
		return std::sqrt(dx * dx + dy * dy);
	}
};

std::istream& operator>> (std::istream &s, Vector &v) {
	return s >> v.x >> v.y;
}

std::ostream& operator<< (std::ostream &s, const Vector &v) {
	s << std::fixed << std::setprecision(6) << v.x << " ";
	return s << std::fixed << std::setprecision(6) << v.y;
}

std::vector<Vector> houses;
std::vector<Vector> wall;
const double eps = 1e-3;
const double EPS = eps;

bool deq(double a, double b) {
	return std::abs(a - b) < eps;
}

struct Dist {
	Vector p;
	double hDist;
	double wDist;

	friend bool operator< (const Dist &a, const Dist &b) {
		if (deq(a.hDist, b.hDist)) {
			if (deq(a.wDist, b.wDist)) {
				return false;
			} else {
				return a.wDist < b.wDist;
			}
		} else {
			return a.hDist < b.hDist;
		}
	}

};
std::vector<Dist> dists; 
uint32_t hCnt, wCnt;

void input() {
	std::cin >> wCnt;
	wall.resize(wCnt);
	for (Vector &v : wall) {
		std::cin >> v;
	}
	std::cin >> hCnt;
	houses.resize(hCnt);
	if (hCnt < 10000) {
		for (Vector &v : houses) {
			std::cin >> v;
		}
	} else {
		uint64_t a, b, c, d, p;
		std::cin >> a >> b >> c >> d >> p;
		for (uint32_t i = 0;i < hCnt;i++) {
			houses[i].x = a;
			houses[i].y = b;

			a = (a * c + d) % p;
			b = (b * d + c) % p;
		}
	}
}

double wallLen = 0;

void calc_dists() {
	for (uint32_t i = 0;i < wCnt;i++) {
		double hDist = 0;
		for (uint32_t j = 0;j < hCnt;j++) {
			hDist += Vector::dist(houses[j], wall[i]);
		}
		dists.push_back(Dist());
		dists.back().p = wall[i];
		dists.back().hDist = hDist;
		dists.back().wDist = wallLen;
		if (i + 1 < wCnt) {
			wallLen += Vector::dist(wall[i], wall[i + 1]);
		}
	}
	wallLen += Vector::dist(wall.back(), wall[0]);
	std::sort(dists.begin(), dists.end());
}

Vector a1, a2;
double d = 1e9;


void solve_group(uint32_t s, uint32_t e) {
	/*<F2>
	if (s - e < 1000) {
		for (uint32_t i = s;i <= e;i++) {
			for (uint32_t j = s;j <= e;j++) {
				double d1 = dists[i].wDist - dists[j].wDist;

				if (d1 < 0) {
					d1 += wallLen;
				}
				double delta1 = std::abs(wallLen - 2 * d1);
				if (delta1 < d) {
					a1 = dists[i].p;
					a2 = dists[j].p;
					d = delta1;
				}
			}
		}
		return;
	}
	*/
	for (uint32_t i = s;i <= e;i++) {
		//std::cerr << "beg " << i << std::endl;
		uint32_t l = 0;
		uint32_t h = e - s + 1;

		while (h - l > 1) {
			uint32_t m = (h + l) / 2;
			uint32_t c = i + m;
			if (c > e) {
				c -= (e - s + 1);
			}

			double curL ;
			curL = dists[c].wDist - dists[i].wDist;
			if (curL + eps < 0) {
				curL += wallLen;
			}
			if (curL < wallLen / 2 + EPS) {
				l = m;
			} else {
				h = m;
			}
		}
		uint32_t c1 = l + i;
		uint32_t c2 = l + i + 1;
		while (c1 > e) {
			c1 -= (e - s + 1);
		}
		while (c2 > e) {
			c2 -= (e - s + 1);
		}
		//std::cerr << ">>> " << c1 << " " << c2 << std::endl;
		Vector p1 = dists[c1].p;
		Vector p2 = dists[c2].p;
		
		double d1 = dists[c1].wDist - dists[i].wDist;
		double d2 = dists[c2].wDist - dists[i].wDist;

		if (d1 < 0) {
			d1 += wallLen;
		}
		if (d2 < 0) {
			d2 += wallLen;
		}

		double delta1 = std::abs(wallLen - 2 * d1);
		double delta2 = std::abs(wallLen - 2 * d2);

		if (delta1 < d) {
			a1 = dists[i].p;
			a2 = p1;
			d = delta1;
		}
		if (delta2 < d) {
			a1 = dists[i].p;
			a2 = p2;
			d = delta2;
		}
	}
}

int main() {
	input();
	calc_dists();


	for (uint32_t i = 0;i < dists.size();i++) {
		//std::cerr << dists[i].p << "; " << dists[i].hDist << " " << dists[i].wDist << "\n";
	}

	for (uint32_t i = 0;i < dists.size();i++) {
		uint32_t j;
		for (j = i;j < dists.size();j++) {
			if (!deq(dists[i].hDist, dists[j].hDist)) {
				break;
			}
		}
		solve_group(i, j - 1);
		i = j - 1;
	}

	std::cout << a1 << "\n" << a2 << "\n";
}
