#include <bits/stdc++.h>
#define endl '\n'
using namespace std;

const int MAX_N = 5e4 + 5;
const int MAX_M = 1e5 + 5;

vector< pair< int, int > > directedEdges;  

class Graph {
public:
	class Node {
	public:
		bool isVisited;
		int in, out;
		vector< int > v, revV;

		bool IsParentOf(const Node *x) {
			if(in <= x->in && out >= x->out) {
				return true;
			}
			else {
				return false;
			}
		}
	};

	int cntNodes, sccInd, dfsTime;
	Node nodes[MAX_N];
	
	void MarkAllNodesUnvisited() {
		for(int i = 1; i <= cntNodes; i++) {
			nodes[i].isVisited = false;
		}
	}

	void Init(int cntNodes) {
		this->cntNodes = cntNodes;
		this->dfsTime = 0;
		MarkAllNodesUnvisited();
	}

	void AddEdge(int x, int y, bool addReversed) {
		nodes[x].v.push_back(y);

		if(addReversed) {
			nodes[y].revV.push_back(x);
		}
	}

	void Dfs(int node, int par) {
		nodes[node].in = dfsTime++;
		nodes[node].isVisited = true;

		for(int x : nodes[node].v) {
			if(x != par) {
				if(find(directedEdges.begin(), directedEdges.end(), make_pair(x, node)) == directedEdges.end()) {
					directedEdges.push_back({ node, x });
				}
				if(!nodes[x].isVisited) {
					Dfs(x, node);
				}
			}
		}

		nodes[node].out = dfsTime - 1;
	}

	void DfsTarjan(int node, stack< int > &st) {
		nodes[node].isVisited = true;

		for(int x : nodes[node].v) {
			if(!nodes[x].isVisited) {
				DfsTarjan(x, st);
			}
		}

		st.push(node);
	}

	void DfsReversedTarjan(int node) {
		nodes[node].isVisited = true;

		for(int x : nodes[node].revV) {
			if(!nodes[x].isVisited) {
				DfsReversedTarjan(x);
			}
		}
	}

	void ComputeSccs() {
		stack< int > st;

		for(int i = 1; i <= cntNodes; i++) {
			if(!nodes[i].isVisited) {
				DfsTarjan(i, st);
			}
		}

		MarkAllNodesUnvisited();
	
		sccInd = 0;
		while(!st.empty()) {
			int curr = st.top();
			st.pop();

			if(!nodes[curr].isVisited) {
				DfsReversedTarjan(curr);	
				sccInd++;
			}
		}
	}
};

int u[MAX_M], v[MAX_M];
Graph g, directedG;

int main() {
	ios_base::sync_with_stdio(false);
	cin.tie(nullptr);

	int n, m;
	cin >> n >> m;

	g.Init(n);
	for(int i = 1; i <= m; i++) {
		cin >> u[i] >> v[i];

		g.AddEdge(u[i], v[i], false);
		g.AddEdge(v[i], u[i], false);
	}
	
	g.Dfs(1, -1);
	
	directedG.Init(n);
	for(auto edge : directedEdges) {
		directedG.AddEdge(edge.first, edge.second, true);
	}
	directedG.ComputeSccs();

	if(directedG.sccInd == 1) {
		cout << 1 << endl;
		for(int i = 1; i <= n; i++) {
			for(int x : directedG.nodes[i].v) {
				cout << i << " " << x << endl;
			}
		}
	}
	else {
		cout << 0 << endl;
	}

	return 0;
}
