#include <bits/stdc++.h>
#define endl '\n'
using namespace std;

const int MAX_N = 5e4 + 5;
const int MAX_M = 1e5 + 5;
	
class Edge {
public:
	int u, v, dir;

	Edge(int u = 0, int v = 0, int dir = -1) : u(u), v(v), dir(dir) {
		
	}
};

Edge edges[MAX_M];
vector< pair< int, int > > directedEdges; 

class Graph {
public:
	class Node {
	public:
		bool isVisited;
		vector< int > v, revV;
	};

	int cntNodes, sccInd;
	Node nodes[MAX_N];
	
	void MarkAllNodesUnvisited() {
		for(int i = 1; i <= cntNodes; i++) {
			nodes[i].isVisited = false;
		}
	}

	void Init(int cntNodes) {
		this->cntNodes = cntNodes;
		MarkAllNodesUnvisited();
	}

	void AddEdge(int x, int y, bool addReversed) {
		nodes[x].v.push_back(y);

		if(addReversed) {
			nodes[y].revV.push_back(x);
		}
	}

	void Dfs(int node, int par) {
		nodes[node].isVisited = true;

		for(int ind : nodes[node].v) {
			int x;

			if(edges[ind].u == node) {
				x = edges[ind].v;
			}
			else {
				x = edges[ind].u;
			}
			
			if(x != par) {
				if(edges[ind].dir == -1) {
					edges[ind].dir = (edges[ind].u == node ? 0 : 1);
					directedEdges.push_back({ node, x });
				}
				if(!nodes[x].isVisited) {
					Dfs(x, node);
				}
			}
		}
	}

	void DfsTarjan(int node, stack< int > &st) {
		nodes[node].isVisited = true;

		for(int x : nodes[node].v) {
			if(!nodes[x].isVisited) {
				DfsTarjan(x, st);
			}
		}

		st.push(node);
	}

	void DfsReversedTarjan(int node) {
		nodes[node].isVisited = true;

		for(int x : nodes[node].revV) {
			if(!nodes[x].isVisited) {
				DfsReversedTarjan(x);
			}
		}
	}

	void ComputeSccs() {
		stack< int > st;

		for(int i = 1; i <= cntNodes; i++) {
			if(!nodes[i].isVisited) {
				DfsTarjan(i, st);
			}
		}

		MarkAllNodesUnvisited();
	
		sccInd = 0;
		while(!st.empty()) {
			int curr = st.top();
			st.pop();

			if(!nodes[curr].isVisited) {
				DfsReversedTarjan(curr);	
				sccInd++;
			}
		}
	}

	void CheckDfs(int node, int remEdge) {
		nodes[node].isVisited = true;

		for(int x : nodes[node].v) {
			if(!(node == edges[remEdge].u && x == edges[remEdge].v) && !(node == edges[remEdge].v && x == edges[remEdge].u)) {
				if(!nodes[x].isVisited) {
					CheckDfs(x, remEdge);
				}
			}
		}
	}
};

int u[MAX_M], v[MAX_M];
Graph g, directedG;

int main() {
	ios_base::sync_with_stdio(false);
	cin.tie(nullptr);

	int n, m;
	cin >> n >> m;

	g.Init(n);
	for(int i = 1; i <= m; i++) {
		int u, v;
		cin >> u >> v;

		edges[i] = Edge(u, v);

		g.AddEdge(u, i, false);
		g.AddEdge(v, i, false);
	}
	
	g.Dfs(1, -1);
	
	directedG.Init(n);
	for(auto edge : directedEdges) {
		directedG.AddEdge(edge.first, edge.second, true);
	}
	directedG.ComputeSccs();

	if(directedG.sccInd == 1) {
		if(n * m <= (int) 1e8) {
			for(int i = 1; i <= m; i++) {
				directedG.MarkAllNodesUnvisited();
				directedG.CheckDfs((edges[i].dir == 0 ? edges[i].u : edges[i].v), i);
				
				if(directedG.nodes[(edges[i].dir == 0 ? edges[i].v : edges[i].u)].isVisited) {
					cout << 2 << endl;
					cout << edges[i].u << " " << edges[i].v << endl;

					for(int j = 1; j <= m; j++) {
						if(i == j) {
							continue;
						}

						if(edges[j].dir == 0) {
							cout << edges[j].u << " " << edges[j].v << endl;
						}
						else {
							cout << edges[j].v << " " << edges[j].u << endl;
						}
					}

					return 0;
				}
			}
		}
		
		cout << 1 << endl;
		for(int i = 1; i <= n; i++) {
			for(int x : directedG.nodes[i].v) {
				cout << i << " " << x << endl;
			}
		}
	}
	else {
		cout << 0 << endl;
	}

	return 0;
}
