#include <bits/stdc++.h>
#define endl '\n'
using namespace std;

const int MAX_N = 5e4 + 5;
const int MAX_M = 1e5 + 5;
	
class Edge {
public:
	bool isDirected, isVisitedRev, isVisited;
	int u, v;

	Edge(int u = 0, int v = 0, bool isDirected = false, bool isVisited = false, bool isVisitedRev = false) : u(u), v(v), isDirected(isDirected), isVisited(isVisited), isVisitedRev(isVisitedRev) {
		
	}

	bool operator< (const Edge &other) {
		if(u == other.u) {
			return v < other.v;
		}
		else {
			return u < other.u;
		}
	}
};

Edge edges[MAX_M];

class Graph {
public:
	class Node {
	public:
		bool isVisited;
		vector< int > v, revV;
	};

	int cntNodes, sccInd;
	Node nodes[MAX_N];
	
	void MarkAllNodesUnvisited() {
		for(int i = 1; i <= cntNodes; i++) {
			nodes[i].isVisited = false;
		}
	}

	void Init(int cntNodes) {
		this->cntNodes = cntNodes;
		MarkAllNodesUnvisited();
	}

	void AddEdge(int x, int y, bool addReversed) {
		nodes[x].v.push_back(y);
		
		if(addReversed) {
			nodes[edges[y].v].revV.push_back(y);
		}
	}

	void Dfs(int node, int par) {
		nodes[node].isVisited = true;

		for(int ind : nodes[node].v) {
			if(edges[ind].isDirected) {
				continue;
			}
			
			int x;

			if(edges[ind].u == node) {
				x = edges[ind].v;
			}
			else {
				x = edges[ind].u;
			}
			
			if(x != par) {
				edges[ind].isDirected = true;
				if(edges[ind].v == node) {
					swap(edges[ind].u, edges[ind].v);
				}
	
				if(!nodes[x].isVisited) {
					Dfs(x, node);
				}
			}
		}
	}

	void DfsTarjan(int node, stack< int > &st) {
		nodes[node].isVisited = true;

		for(int ind : nodes[node].v) {
			int x = edges[ind].v;

			if(!nodes[x].isVisited) {
				DfsTarjan(x, st);
				edges[ind].isVisited = true;
			}
		}

		st.push(node);
	}

	void DfsReversedTarjan(int node) {
		nodes[node].isVisited = true;

		for(int ind : nodes[node].revV) {
			if(!edges[ind].isVisited) {
				continue;
			}

			int x = edges[ind].u;

			if(!nodes[x].isVisited) {
				DfsReversedTarjan(x);
				edges[ind].isVisitedRev = true;
			}
		}

		for(int ind : nodes[node].revV) {
			if(edges[ind].isVisited) {
				continue;	
			}
			
			int x = edges[ind].u;

			if(!nodes[x].isVisited) {
				DfsReversedTarjan(x);
				edges[ind].isVisitedRev = true;
			}
		}
	}

	void ComputeSccs() {
		stack< int > st;

		for(int i = 1; i <= cntNodes; i++) {
			if(!nodes[i].isVisited) {
				DfsTarjan(i, st);
			}
		}

		MarkAllNodesUnvisited();
	
		sccInd = 0;
		while(!st.empty()) {
			int curr = st.top();
			st.pop();

			if(!nodes[curr].isVisited) {
				DfsReversedTarjan(curr);	
				sccInd++;
			}
		}
	}

	void CheckDfs(int node, int remEdge) {
		nodes[node].isVisited = true;

		for(int ind : nodes[node].v) {
			int x = edges[ind].v;
			
			if(ind != remEdge) {
				if(!nodes[x].isVisited) {
					CheckDfs(x, remEdge);
				}
			}
		}
	}
};

int u[MAX_M], v[MAX_M];
Graph g, directedG;

int main() {
	ios_base::sync_with_stdio(false);
	cin.tie(nullptr);

	int n, m;
	cin >> n >> m;

	g.Init(n);
	for(int i = 1; i <= m; i++) {
		int u, v;
		cin >> u >> v;

		edges[i] = Edge(u, v);
	}

	for(int i = 1; i <= m; i++) {
		g.AddEdge(edges[i].u, i, false);
		g.AddEdge(edges[i].v, i, false);
	}

	g.Dfs(1, -1);
	
	sort(edges + 1, edges + 1 + m);

	directedG.Init(n);	
	for(int i = 1; i <= m; i++) {
		directedG.AddEdge(edges[i].u, i, true);
	}
	
	directedG.ComputeSccs();

	if(directedG.sccInd == 1) {
		int cnt = 0;
		for(int i = 1; i <= m; i++) {
			if(!edges[i].isVisited && !edges[i].isVisitedRev) {
				cout << 2 << endl;
				cout << edges[i].u << " " << edges[i].v << endl;

				for(int j = 1; j <= m; j++) {
					if(i == j) {
						continue;
					}

					cout << edges[j].u << " " << edges[j].v << endl;
				}
				return 0;
			}
		}

		int ind, maxIn = 0;
		for(int i = 1; i <= n; i++) {
			if(directedG.nodes[i].revV.size() > maxIn) {
				maxIn = directedG.nodes[i].revV.size();	
				ind = i;
			}
		}

		for(int i = 1; i <= m; i++) {
			if(cnt < 10 && edges[i].v == ind && (!edges[i].isVisited || !edges[i].isVisitedRev)) {
				directedG.MarkAllNodesUnvisited();
				directedG.CheckDfs(edges[i].u, i);
			
				if(directedG.nodes[edges[i].v].isVisited) {
					cout << 2 << endl;
					cout << edges[i].u << " " << edges[i].v << endl;

					for(int j = 1; j <= m; j++) {
						if(i == j) {
							continue;
						}

						cout << edges[j].u << " " << edges[j].v << endl;
					}
					return 0;
				}

				cnt++;
			}
		}
		
		cout << 1 << endl;
		for(int i = 1; i <= m; i++) {
			cout << edges[i].u << " " << edges[i].v << endl;
		}
	}
	else {
		cout << 0 << endl;
	}

	return 0;
}
