#include <bits/stdc++.h>
#define endl '\n'

using namespace std;
template<class T, class T1> void chkmax(T &x, const T1 &y) { if(x < y) x = y; }
template<class T, class T1> void chkmin(T &x, const T1 &y) { if(x > y) x = y; }
const int MAXN = (1 << 20);

struct segment_tree
{
	int tr[4 * MAXN];
	int bal[4 * MAXN];

	void init(int l, int r, int idx)
	{
		if(l == r)
		{
			bal[idx] = 0;
			tr[idx] = 0;
			return;
		}
	
		int mid = (l + r) >> 1;
		init(l, mid, 2 * idx + 1);
		init(mid + 1, r, 2 * idx + 2);
		
		bal[idx] = 0;
		tr[idx] = 0;	
	}
	
	void add(int ql, int qr, int c, int l, int r, int idx)
	{
		if(ql > r || qr < l || ql > qr || l > r) return;
	
		if(l <= ql && r <= qr) 
		{
			bal[idx] += c;
			if(bal[idx] > 0) tr[idx] = (r - l + 1);
			else if(l != r) tr[idx] = tr[2 * idx + 1] + tr[2 * idx + 2];
			else tr[idx] = 0;
			return;
		}
	
		int mid = (l + r) >> 1;
		if(ql <= mid) add(ql, qr, c, l, mid, 2 * idx + 1);
		if(qr >= mid + 1) add(ql, c, qr, mid + 1, r, 2 * idx + 2);
		
		if(bal[idx] == 0) tr[idx] = tr[2 * idx + 1] + tr[2 * idx + 2];
		else tr[idx] = (r - l + 1);		
	}
};

int n, m;
vector<int> G[MAXN], adj[MAXN];
vector<pair<int, int> > li;

void read()
{
	cin >> n >> m;
	for(int i = 0; i < m; i++)
	{
		int u, v;
		cin >> u >> v;
		G[u].push_back(v);
		G[v].push_back(u);
		li.push_back({min(u, v), max(u, v)});
	}
}

bool visited[MAXN];
set<pair<int, int> > other;
int tr_sz[MAXN];

void build_tree(int u)
{
	visited[u] = 1;
	tr_sz[u] = 1;
	for(int v: G[u])
		if(!visited[v])
		{
			adj[u].push_back(v);
			adj[v].push_back(u);
			build_tree(v);
			tr_sz[u] += tr_sz[v];
		}
		else 
			other.insert({min(u, v), max(u, v)});
}

int head[MAXN];
int st[MAXN], en[MAXN], dfs_time;
int dep[MAXN], par[MAXN];

void decompose(int u, int pr, int head)
{
	::head[u] = head;
	st[u] = ++dfs_time;
	dep[u] = dep[pr] + 1;
	par[u] = pr;
	
	pair<int, int> mx = {-1, -1};
	for(int v: adj[u])
		if(pr != v)
			mx = max(mx, make_pair(tr_sz[v], v));
			
	if(mx.second != -1) 
		decompose(mx.second, u, head);
		
	for(int v: adj[u])
		if(v != mx.second && v != pr)
			decompose(v, u, v);
	
	en[u] = dfs_time;
}	

segment_tree t;

void add(int u, int anc, int val)
{
	if(dep[anc] > dep[u]) swap(u, anc);
	while(st[anc] < st[u])
	{
		t.add(max(st[anc] + 1, st[head[u]]), st[u], val, 1, dfs_time, 0);
		u = par[head[u]];
	}
	
	t.add(st[anc], st[anc], val, 1, dfs_time, 0); 
} 

void solve()
{
	build_tree(1);
	decompose(1, 1, 1);

	t.init(1, dfs_time, 0);

	for(auto it: other)
		add(it.first, it.second, 1);
			
	if(t.tr[0] != n)
	{
		cout << 0 << endl;
		return;
	}
	
	for(auto it: other)
	{
		add(it.first, it.second, -1);
		
		if(t.tr[0] == n)
		{
			cout << 2 << endl;
			cout << it.first << " " << it.second << endl;
			for(auto it2: li)
			{
				if(it == it2) continue; 
				auto p = it2;
				if(dep[p.first] > dep[p.second]) swap(p.first, p.second);
				if(par[p.first] == p.second || par[p.second] == p.first) swap(p.first, p.second);
				cout << p.first << " " << p.second << endl;	
			}
			
			return;
		}
		
		add(it.first, it.second, 1);	
	}
	
	cout << 1 << endl;
	for(auto it2: li)
	{
		auto p = it2;
		if(dep[p.first] > dep[p.second]) swap(p.first, p.second);
		if(par[p.first] == p.second || par[p.second] == p.first) swap(p.first, p.second);
		cout << p.first << " " << p.second << endl;	
	}
}

int main()
{
	ios_base::sync_with_stdio(false);
	cin.tie(NULL);
	
	read();
	solve();
	return 0;
}

